#!/bin/bash
###############################################################################
#                              RUN EC_EARTH
###############################################################################
#PBS -N run_ecearth_coupled
#PBS -l size=56
#PBS -o /tmp/proj/cli055/models/ecearth/v3-r719/scripts/
#PBS -e /tmp/proj/cli055/models/ecearth/v3-r719/scripts/
#PBS -l walltime=00:30:00
#PBS -A cli055
#PBS -l gres=widow2%widow3
#

# setup the grib_api env.
PATH=/tmp/proj/cli055/opt/grib_api-1.9.16/intel-12.1/bin:$PATH

set -xuve

# =============================================================================
# *** BEGIN User configuration
# =============================================================================

# -----------------------------------------------------------------------------
# Switch between LOW, STANDARD and HIGH RESOLUTION by setting:
#
# --------------------------------------------------
#   Parameter          lowres     stdres    highres
# --------------------------------------------------
#   ifs_grid           T159L62      T255       T799
#   nem_grid          ORCA1L46  ORCA1L46 ORCA025L46
#   ifs_time_step_sec     3600      2700        720
#   nem_time_step_sec     3600      3600       1200
# --------------------------------------------------
#
# Change also:
#   exp_name
#   run_start_date (lo:1990/std:1996/hi:1996)
#   ifs_numproc
#   nem_numproc
#   run_dir
# and make sure to use the appropriate NEMO executable
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# *** General configuration
# -----------------------------------------------------------------------------
# Experiment name (exactly 4 letters!)
exp_name=ECE3

# Simulation start and end date. Use any (reasonable) syntax you want.
run_start_date="1990-01-01"
run_end_date="${run_start_date} + 5 days"

# Set $force_run_from_scratch to 'true' if you want to force this run to start
# from scratch, possibly ignoring any restart files present in the run
# directory. Leave set to 'false' otherwise.
# NOTE: If set to 'true' the run directory $run_dir is cleaned!
force_run_from_scratch=false

# Resolution
ifs_grid=T255L62
nem_grid=ORCA1L46
lim=LIM2

# Restart frequency. Use any (reasonable) number and time unit you want.
# For runs without restart, leave this variable empty
rst_freq="1 month"

# Coupling frequency [hrs]
cpl_freq_hrs=3

# Directories
ecearth_base_dir=/tmp/proj/cli055/models/ecearth/v3-r719
start_dir=${PWD}
ctrl_file_dir=${ecearth_base_dir}/setup/ctrl
run_dir=/tmp/work/$USER/test/run_${ifs_grid}_${nem_grid}_${PBS_JOBID}
ini_data_dir=${ecearth_base_dir}/inidata
bin_dir=${ecearth_base_dir}/bin

# Architecture
build_arch=eexcon

# This file is used to store information about restarts
ece_info_file="ece.info"

# -----------------------------------------------------------------------------
# *** Read platform dependent configuration
# -----------------------------------------------------------------------------
grib_api_path=/tmp/proj/cli055/opt/grib_api-1.9.16/intel-12.1

export GRIB_DEFINITION_PATH=${grib_api_path}/share/definitions
export GRIB_SAMPLES_PATH=${grib_api_path}/share/grib_api/ifs_samples/grib1
export GRIB_BIN_PATH=${grib_api_path}/bin

# -----------------------------------------------------------------------------
# *** IFS configuration
# -----------------------------------------------------------------------------

case ${ifs_grid} in
     T159L*)  ifs_time_step_sec=3600
     ;;
     T255L*)  ifs_time_step_sec=2700
     ;;
     T799L*)  ifs_time_step_sec=720
     ;;
     *) error "Unsupported grid type: ${ifs_grid}"
     ;;
esac

ifs_output_freq=$(( 6 * 3600 / ifs_time_step_sec ))
ifs_di_freq=$(( 1 * 3600 / ifs_time_step_sec ))
ifs_ddh_freq=$(( 1 * 3600 / ifs_time_step_sec ))

ifs_numproc=32

ifs_exe_file=${bin_dir}/ifsmaster-${build_arch}

ifs_lastout=true

nprtrv=1
# -----------------------------------------------------------------------------
# *** NEMO/LIM configuration
# -----------------------------------------------------------------------------

case ${nem_grid} in
     ORCA1L46)   nem_time_step_sec=3600
     case ${lim} in
          lim2) nem_exe_file=${bin_dir}/ORCA1_LIM2/nemo.exe ;;
          lim3) nem_exe_file=${bin_dir}/ORCA1_LIM3/nemo.exe ;;
     esac 
     ;;
     ORCA025L46) nem_time_step_sec=1200
     case ${lim} in
          lim2) nem_exe_file=${bin_dir}/ORCA025_LIM2/nemo.exe ;;
          lim3) nem_exe_file=${bin_dir}/ORCA025_LIM3/nemo.exe ;;
     esac 
     ;;
     *) error "Unsupported grid type: ${nem_grid}"
    ;;
esac

lim_time_step_sec=3600

nem_numproc=16
advsch=tvd
rstctl=1

# -----------------------------------------------------------------------------
# *** OASIS configuration
# -----------------------------------------------------------------------------

oas_namcut_script=${ecearth_base_dir}/sources/util/namcouple-split/namsplit.pl

oas_numproc=7

oas_exe_file=${bin_dir}/oasis3.MPI1.x

# =============================================================================
# *** END of User configuration
# =============================================================================

# =============================================================================
# *** This is where the code begins ...
# =============================================================================

# -----------------------------------------------------------------------------
# *** Define some helper functions
# -----------------------------------------------------------------------------

function info
{
    echo "*II* $@"
}

function error
{
    echo "*EE* $@"
    exit 1
}

function cleanup
{
    # Cleanup function, triggered by the traps set in the run-script
    set +u

    if [ -r ${tempfile} ]
    then
        rm -f ${tempfile}
    fi

    if [ -r ${stdout_file} -a -d ${run_dir} ]
    then
        cp ${stdout_file} ${run_dir}
    fi
}

# -----------------------------------------------------------------------------
# *** Make sure to clean up on exit
# -----------------------------------------------------------------------------

trap 'cleanup' EXIT SIGHUP SIGINT SIGTERM

# -----------------------------------------------------------------------------
# *** Create the run dir if necessary and go there
#     Everything is done from here.
# -----------------------------------------------------------------------------
if [ ! -d ${run_dir} ]
then
    mkdir -p ${run_dir}
fi
cd ${run_dir}

# -----------------------------------------------------------------------------
# *** Determine the time span of this run and whether it's a restart leg
# -----------------------------------------------------------------------------

# Regularise the format of the start and end date of the simulation
run_start_date=$(date -uR -d "${run_start_date}")
run_end_date=$(date -uR -d "${run_end_date}")

# Check for restart information file and set the current leg start date
#   Ignore restart information file if force_run_from_scratch is true
if ! $force_run_from_scratch && [ -r ${ece_info_file} ]
then
    leg_is_restart=true
    . ./${ece_info_file}
    leg_start_date=${leg_end_date}
    leg_number=$((leg_number+1))
else
    leg_is_restart=false
    leg_start_date=${run_start_date}
    leg_number=1
fi

# Compute the end date of the current leg
if [ -n "${rst_freq}" ]
then
    leg_end_date=$(date -uR -d "${leg_start_date} + ${rst_freq}")
else
    leg_end_date=${run_end_date}
fi

if [ $(date -d "${leg_end_date}" +%s) -gt $(date -d "${run_end_date}" +%s) ]
then
    leg_end_date=${run_end_date}
fi

# Some time variables needed later
leg_length_sec=$(( $(date -d "${leg_end_date}" +%s) - $(date -d "${leg_start_date}" +%s) ))
leg_start_sec=$(( $(date -d "${leg_start_date}" +%s) - $(date -d "${run_start_date}" +%s) ))
leg_end_sec=$(( $(date -d "${leg_end_date}" +%s) - $(date -d "${run_start_date}" +%s) ))
leg_start_date_yyyymmdd=$(date -u -d "${leg_start_date}" +%Y%m%d)
leg_start_date_yyyy=$(date -u -d "${leg_start_date}" +%Y)
leg_end_date_yyyy=$(date -u -d "${leg_end_date}" +%Y)

# Check whether there's actually time left to simulate - exit otherwise
if [ ${leg_length_sec} -le 0 ]
then
    info "Leg start date equal to or after end of simulation."
    info "Nothing left to do. Exiting."
    exit 0
fi

# -----------------------------------------------------------------------------
# *** Prepare the run directory for a run from scratch
# -----------------------------------------------------------------------------
if ! $leg_is_restart
then
    # -------------------------------------------------------------------------
    # *** This is no restart, hence, clean the run dir and link the executables
    # -------------------------------------------------------------------------
    rm -fr ${run_dir}/*

    ln -s ${ifs_exe_file}
    ln -s ${nem_exe_file}
    ln -s ${oas_exe_file}

    # -------------------------------------------------------------------------
    # *** Files needed for IFS (linked)
    # -------------------------------------------------------------------------

    # Initial data
    ln -s \
    ${ini_data_dir}/ifs/${ifs_grid}/${leg_start_date_yyyymmdd}/ICMGGECE3INIUA \
                                                        ICMGG${exp_name}INIUA
    ln -s \
    ${ini_data_dir}/ifs/${ifs_grid}/${leg_start_date_yyyymmdd}/ICMGGECE3INIT \
                                                        ICMGG${exp_name}INIT
    ln -s \
    ${ini_data_dir}/ifs/${ifs_grid}/${leg_start_date_yyyymmdd}/ICMSHECE3INIT \
                                                        ICMSH${exp_name}INIT
    # Other stuff
    ln -s ${ini_data_dir}/ifs/postins
    ln -s ${ini_data_dir}/ifs/dirlist
    ln -s ${ini_data_dir}/ifs/rtables/* .

    # Run-off file
    ln -s ${ini_data_dir}/ifs/${ifs_grid}/runoff_maps.txt

    # -------------------------------------------------------------------------
    # *** Files needed for NEMO (linked)
    # -------------------------------------------------------------------------

    # Various stuff
    ln -s ${ini_data_dir}/nemo/${nem_grid}/bathy_meter.nc
    ln -s ${ini_data_dir}/nemo/${nem_grid}/coordinates.nc
    ln -s ${ini_data_dir}/nemo/${nem_grid}/ahmcoef

    # Initial data
    ln -s ${ini_data_dir}/nemo/${nem_grid}/temperature_*.nc .
    ln -s ${ini_data_dir}/nemo/${nem_grid}/salinity_*.nc .

    # IOM files
    ln -s ${ctrl_file_dir}/xmlio_server.def
    ln -s ${ctrl_file_dir}/iodef.xml

    # -------------------------------------------------------------------------
    # *** Files needed for OASIS (linked)
    # -------------------------------------------------------------------------

    # Name table file
    ln -s ${ini_data_dir}/oasis/cf_name_table.txt

    # Grid definition files
    ln -s ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/areas.nc
    ln -s ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/grids.nc
    ln -s ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/masks.nc

    # Weight files
    case ${ifs_grid} in
        T159L*)  oas_agrd=080
                  ;;
        T255L*)  oas_agrd=128
                  ;;
        T799L*)  oas_agrd=400
                  ;;
        *)       error "Unsupported grid type: ${ifs_grid}"
                  ;;
    esac

    case ${nem_grid} in
        ORCA1L46)   oas_ogrd=O1t0
                    ;;
        ORCA025L46) oas_ogrd=Ot25
                    ;;
        *)          error "Unsupported grid type: ${nem_grid}"
                    ;;
    esac

    for (( n=0; n<oas_numproc; n++ ))
    do
        ln -s \
        ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/rmp_A${oas_agrd}_to_${oas_ogrd}_GAUSWGT.nc \
                                                 rmp_A${oas_agrd}_to_${oas_ogrd}_GAUSWGT_${n}.nc
        ln -s \
        ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/rmp_${oas_ogrd}_to_L${oas_agrd}_GAUSWGT.nc \
                                                 rmp_${oas_ogrd}_to_L${oas_agrd}_GAUSWGT_${n}.nc
    done

    # Restart files for the coupling fields
    cpl_fields="TAUX_OCE TAUY_OCE TAUX_ICE TAUY_ICE \
                QS___OCE QS___ICE QNS__OCE QNS__ICE DQDT_ICE \
                PRCP_LIQ PRCP_SOL EVAP_TOT EVAP_ICE RNF__OCE \
                O_SSTSST O_TepIce O_AlbIce OIceFrac O_IceTck O_SnwTck"

    for f in ${cpl_fields}
    do
        cp ${ini_data_dir}/oasis/${ifs_grid}-${nem_grid}/rst/$f .
    done

fi # ! $leg_is_restart

# -----------------------------------------------------------------------------
# *** Create some control files
# -----------------------------------------------------------------------------

# IFS, NEMO, LIM namelist and OASIS namcouple files
. ${ctrl_file_dir}/namelist.ifs.sh   > fort.4
. ${ctrl_file_dir}/namelist.${lim}.sh  > namelist_ice
. ${ctrl_file_dir}/namcouple.sh      > namcouple

case ${nem_grid} in
        ORCA1L46)   . ${ctrl_file_dir}/namelist.nemo-orca1.sh    > namelist
                    ;;
        ORCA025L46) . ${ctrl_file_dir}/namelist.nemo-orca025.sh  > namelist
                    ;;
        *)          error "Unsupported grid type: ${nem_grid}"
                    ;;
esac

# Split OASIS namcouple file (if necessary)
if (( oas_numproc == 1 ))
then
    ln -s namcouple namcouple_0
elif [ -n "$oas_namcut_script" -a -x $oas_namcut_script ]
then
    ${oas_namcut_script} ${oas_numproc} namcouple
else
    error "Couldn't run script to split the OASIS namcouple file (\$oas_namcut_script)."
fi

# -----------------------------------------------------------------------------
# *** IFS climatology data
# -----------------------------------------------------------------------------
icmclfile=${ini_data_dir}/ifs/${ifs_grid}/climate/ICMCL
tempfile=tmp.$$

# Create data for december, the year before the leg starts
grib_set \
-s dataDate=$(printf "%04d" $((leg_start_date_yyyy-1)))1215 ${icmclfile}-12 \
                                                       ICMCL${exp_name}INIT

# Create data for all month in the years of the leg
for y in $(seq ${leg_start_date_yyyy} ${leg_end_date_yyyy})
do
    yy=$(printf "%04d" $y)
    for m in $(seq 1 12)
    do
        mm=$(printf "%02d" $m)
        grib_set -s dataDate=${yy}${mm}15 ${icmclfile}-${mm} ${tempfile}
        cat ${tempfile} >> ICMCL${exp_name}INIT
    done
done

# Create data for january, the year after the leg ends
grib_set -s dataDate=$(printf "%04d" $((leg_end_date_yyyy+1)))0115 ${icmclfile}-01 \
                                                                          ${tempfile}
cat ${tempfile} >> ICMCL${exp_name}INIT

# Clean up
rm -f ${tempfile}

# -----------------------------------------------------------------------------
# *** Link the appropriate NEMO restart files of the previous leg
# -----------------------------------------------------------------------------
if $leg_is_restart
then
    ns=$(printf %08d $(( leg_start_sec / nem_time_step_sec )))
    for n in $(seq 0 $((nem_numproc-1)))
    do
        np=$(printf %04d ${n})
        ln -fs ${exp_name}_${ns}_restart_${np}.nc restart_${np}.nc
        ln -fs ${exp_name}_${ns}_restart_ice_${np}.nc restart_ice_in_${np}.nc
    done
fi

# -----------------------------------------------------------------------------
# *** Remove some OASIS files of the previous leg
# -----------------------------------------------------------------------------
if $leg_is_restart
then
    rm -f anaisout_*
fi

# -----------------------------------------------------------------------------
# *** Start the run
# -----------------------------------------------------------------------------
export DR_HOOK_IGNORE_SIGNALS='-1'
export OASIS3=yes
export OASIS3DEBUGLEVEL=2

### --- PLATFORM DEPENDENT CODE ---
ulimit -s unlimited
### --- END of PLATFORM DEPENDENT CODE ---

t1=$(date +%s)

### --- PLATFORM DEPENDENT CODE ---
aprun -n ${oas_numproc} ${oas_exe_file} : \
      -n ${ifs_numproc} ${ifs_exe_file} -v ecmwf -e ${exp_name} : \
      -n ${nem_numproc} ${nem_exe_file}
### --- END of PLATFORM DEPENDENT CODE ---

t2=$(date +%s)

tr=$(date -d "0 -$t1 sec + $t2 sec" +%T)

# -----------------------------------------------------------------------------
# *** Write the restart control file
# -----------------------------------------------------------------------------
echo "#"                                             | tee -a ${ece_info_file}
echo "# Finished leg at `date '+%F %T'` after ${tr} (hh:mm:ss)" \
                                                     | tee -a ${ece_info_file}
echo "leg_number=${leg_number}"                      | tee -a ${ece_info_file}
echo "leg_start_date=\"${leg_start_date}\""          | tee -a ${ece_info_file}
echo "leg_end_date=\"${leg_end_date}\""              | tee -a ${ece_info_file}

# -----------------------------------------------------------------------------
# *** Platform dependent finalising of the run
# -----------------------------------------------------------------------------
finalise

exit 0
